
/*
 * The example program tests the excitation portion of the RadioProcessor.
 * This program will produce a 1MHz signal on the oscilloscope that is on
 * for 10 microseconds, off for 1 milisecond, and the repeat this pattern.
 *
 * SpinCore Technologies, Inc.
 * www.spincore.com
 * $Date: 2006/05/01 17:54:22 $
 */

#include <stdio.h>
#include "spinapi.h"

#define CLOCK_FREQ 75.0

// User friendly names for the control bits
#define TX_ENABLE 1
#define TX_DISABLE 0
#define PHASE_RESET 1
#define NO_PHASE_RESET 0
#define DO_TRIGGER 1
#define NO_TRIGGER 0

int main(int argc, char **argv)
{
	int start;
    pb_set_debug(1);
    
    printf ("Using spinapi library version %s\n", pb_get_version());
    if(pb_init()) {
        printf ("Error initializing board: %s\n", pb_get_error());    
	    while (getchar() != '\n');
        return -1;
    }
    
    pb_set_clock(75.0);
    
   	pb_select_dds(0); //Select DDS0
   	
	pb_start_programming(FREQ_REGS);	
	    pb_set_freq(10.0*MHz);
	pb_stop_programming();

	pb_start_programming(TX_PHASE_REGS);	
        pb_set_phase(0.0);
	pb_stop_programming();
	
	pb_set_amp(1.0,0);
    		
	pb_select_dds(1); //Select DD1

	pb_start_programming(FREQ_REGS);	
	    pb_set_freq(10.0*MHz);
	pb_stop_programming();

	pb_start_programming(TX_PHASE_REGS);	
      	pb_set_phase(90.0);
	pb_stop_programming();
	
	
	
	pb_set_amp(1.0,0);

//	SPINCORE_API int pb_inst_dds2(int freq0, int phase0, int amp0, int dds_en0, int phase_reset0, int freq1, int phase1, int amp1, int dds_en1, int phase_reset1, int flags, int inst, int inst_data, double length)

	pb_start_programming(PULSE_PROGRAM);
	    start = pb_inst_dds2(0, 0, 0, TX_ENABLE,  NO_PHASE_RESET, 0, 0, 0, TX_DISABLE, PHASE_RESET, 0xfff, CONTINUE, 0, 10.0*us);
                pb_inst_dds2(0, 0, 0, TX_DISABLE, PHASE_RESET, 0, 0, 0, TX_ENABLE,  NO_PHASE_RESET, 0x000, BRANCH, start, 10.0*us);
                
       int isr1 = pb_inst_dds2(0,0, 0,1, 0, 0, 0, 0,0, 1, 0xfff, CONTINUE, 0, 20.0*us);
                  pb_inst_dds2(0,0, 0,0, 0, 0, 0, 0,1, 0, 0x000, BRANCH, isr1, 20.0*us);
	pb_stop_programming();
	
	pb_set_isr(0,isr1);               //Program IRQ0 ISR
	pb_set_irq_enable_mask(0xF);      //Interrupt enable mask.
	//pb_set_irq_immediate_mask(0xA); //Set which IRQs are immediate IRQs (immediately perform ISR)
	
	pb_start();
    while (getchar() != '\n');
    pb_stop(); 
	return 0;
}

